@@ -16,14 +16,16 @@ import scipy.stats as scs
 # FUCNIONES para el analisis exploratorio de datos
 #------------------------------------------------------------
 
-def simetria(column):
+# Hacer este mtodo privado
+
+def __simetria(column):
   kurt = 0.00
   skew = 0.00
   kurt = stats.kurtosis(column)
   skew = stats.skew(column)
   return kurt, skew
 
-def graficosxy(df):
+def __graficosxy(df):
   df.plot(
       kind='line',
       subplots=True,
@@ -33,7 +35,7 @@ def graficosxy(df):
   plt.tight_layout()
   plt.show()
   
-def histograma(df):
+def __histograma(df):
   
   nrows = len(df.columns)
   npar = nrows%2
@@ -42,6 +44,7 @@ def histograma(df):
   nrows = nrows//2
   nh = nrows * 3
   
+    
   # Setting up the figure and axes
   fig, axs = plt.subplots(nrows, 2, figsize=(8,nh))
   plt.subplots_adjust(hspace=0.5,wspace=0.3)
@@ -50,8 +53,12 @@ def histograma(df):
   columns = df.columns
 
   for i, column in enumerate(columns):
-               
-        ax = axs[i//2, i%2]
+        
+        if(nrows==1):
+           ax = axs[i]
+        else:
+           ax = axs[i//2, i%2]
+        
         # Plot histogram and KDE
         ax = sns.histplot(df[column], kde=True, ax=ax, color='#b5b5ff', bins=15, line_kws={'linewidth': 1})
         try:
@@ -74,7 +81,7 @@ def histograma(df):
   plt.tight_layout()
   plt.show()
 
-def dispersion_index(df):
+def __dispersion_index(df):
   
   nrows = len(df.columns)
   npar = nrows%2
@@ -83,7 +90,7 @@ def dispersion_index(df):
   nrows = nrows//2
   nh = nrows * 3
   ncols = len(df.columns)
-  
+   
   # Setting up the figure and axes
   fig, axs = plt.subplots(nrows, 2, figsize=(8,nh))
   plt.subplots_adjust(hspace=0.5,wspace=0.5)
@@ -93,7 +100,10 @@ def dispersion_index(df):
 
   for i, column_name in enumerate(columns):
                
-      ax = axs[i//2, i%2]
+      if(nrows==1):
+           ax = axs[i]
+      else:
+           ax = axs[i//2, i%2]
                   
       row_index = i // ncols
       col_index = i % ncols
@@ -108,28 +118,39 @@ def dispersion_index(df):
   plt.tight_layout()
   plt.show() 
  
-def areaTrazadoBoxPlot(qcol):
+def __areaTrazadoBoxPlot(qcol):
   # Setea el area de dibujo
   nw = 0
   nh = 0
-  if (qcol<=2):
-    nw= 3
+  if (qcol==1):
+     nw= 1.5
+     nh= 4.5       
+  elif (qcol > 1 and qcol<=3):
+    nw= 3.5
     nh= 6
-  elif (qcol > 2 and qcol<=4):
+  elif (qcol > 3 and qcol<=5):
     nw= 8
     nh= 6
-  elif (qcol > 4):
+  elif (qcol > 5 and qcol<=8):
     nw= 14
     nh= 6
+  elif (qcol > 8):
+    nw= 18
+    nh= 8
 
   plt.rcParams['figure.figsize']=(nw,nh)
   return (nw,nh)
   
-# inputs, lista de campos numericos
-def boxplot2(df, inputs):
+def __boxplot(df, inputs):
     num_inputs = len(inputs)
-    fig, axs = plt.subplots(1, num_inputs, figsize=areaTrazadoBoxPlot(num_inputs))
-    axs = np.array(axs)
+    fig, axs = plt.subplots(1, num_inputs, figsize=__areaTrazadoBoxPlot(num_inputs))
+    axs = np.array(axs)   
+    len_mayor_longitud = len(max(inputs, key=len))
+    rotar=False
+    
+    if (len_mayor_longitud>=12):
+        rotar=True
+    
     for i, (ax, curve) in enumerate(zip(axs.flat, inputs), 1):
         sns.boxplot(y=df[curve], ax=ax, color='cornflowerblue', showmeans=True,
                 meanprops={"marker":"o",
@@ -139,8 +160,10 @@ def boxplot2(df, inputs):
                flierprops={'marker':'o',
                           'markerfacecolor':'darkgreen',
                           'markeredgecolor':'darkgreen'})
-
-        ax.set_title(inputs[i-1])
+        if(rotar):
+          ax.set_title(inputs[i-1], rotation=15)
+        else:
+           ax.set_title(inputs[i-1])
         ax.set_ylabel('')
 
     plt.subplots_adjust(hspace=0.15, wspace=1.25)
@@ -150,71 +173,128 @@ def boxplot2(df, inputs):
 # EDA
 #------------------------------------------------------------
 
-def categorical(df):
+def categorical(df,varint=False):
   
   import warnings 
   warnings.filterwarnings('ignore')
   from tabulate import tabulate
+  from datetime import datetime
+  
+  if df.shape[1] < 1:
+     print("El DataFrame debe tener al menos una variable.")
+     return
 
   colCat = []
   oIndex = []
   
-  # Se excluyen variables float y fecha en cualquier formato
-  for i in df.columns:
-    if (df[i].dtype != 'float64' and not pd.api.types.is_datetime64_any_dtype(df[i])):
-        colCat.append(i)      
+  # Slo variables string o integer
+  
+  if (varint==True):
+      for i in df.columns:
+        if df[i].dtype.kind in {'O', 'i'}:
+            colCat.append(i)
+  else:
+     for i in df.columns:
+        if df[i].dtype.kind in 'O':
+            colCat.append(i)
+
+  # Convierte todas las variables a string
+  df = df[colCat]          
+  df = df.astype(str)
 
   # Calculo del numero de filas necesario basado en el numero de columnas categoricas
-  num_col = 3  # Numero de columnas por fila en la grilla
+  num_col = 2  # Numero de columnas por fila en la grilla
   num_filas = np.ceil(len(colCat) / num_col).astype(int)
 
   # Crear una figura y un conjunto de subgraficos
-  fig, axes = plt.subplots(num_filas, num_col, figsize=(12, 3*num_filas))
+  fig, axes = plt.subplots(num_filas, num_col, figsize=(8, 3*num_filas))
 
   # Aplanar el array de axes para facilitar su uso en un loop
   axes = axes.flatten()
+ 
+  print('\033[1mANALISIS DE VALORES NULOS\033[0m')
+  df_info = pd.DataFrame({'type': df.dtypes,
+                        'nulos': df.isnull().sum(),
+                        'no_nulos': df.notnull().sum()})
+  table = tabulate(df_info, headers='keys', tablefmt='fancy_grid')
+  print(table)
+  print('') 
+  
 
   print('')
   print('\033[1mMODA POR CATEGORIAS\033[0m')
-  dfModas = df[colCat].mode().T
+  dfModas = df.mode().T
   dfModas['Moda'] = dfModas[0]
   table = tabulate(dfModas['Moda'],  tablefmt='fancy_grid')
   print(table)
-  print('')
+  print('') 
 
   print('\033[1mGRAFICOS DE BARRAS\033[0m')
   print('')
+  
+  arrayFrecuencia = []
+  tofile = False
+  nRows = 0
 
   for i, columna in enumerate(colCat):
     # Contar la frecuencia de cada categoria en la columna actual
-    conteo = df[columna].value_counts()
+    frecuencia = df[columna].value_counts()
+    arrayFrecuencia.append(frecuencia)
     
+    # nRows debe ser igual al numero de filas de frecuencia
+    nRows += len(frecuencia)
+
     # Crear el grafico de barras en el subplot correspondiente
-    conteo.plot(kind='bar', ax=axes[i])
+    frecuencia.plot(kind='bar', ax=axes[i])
     axes[i].set_ylabel('Frecuencia')
     axes[i].set_xlabel('')
     axes[i].set_title(columna)
     
     # Verificar la cantidad de etiquetas del eje X
-    if len(conteo) > 20:
-        # Si hay mas de 20 etiquetas, ocultarlas porque se vuelve ilegible
+    if len(frecuencia) > 18:
+        # Si hay mas de 18 etiquetas, ocultarlas porque se vuelve ilegible
         axes[i].set_xticklabels([])
     else:
-        # Si hay 20 etiquetas o menos, rotarlas para mejor legibilidad
+        # Si hay 18 etiquetas o menos, rotarlas para mejor legibilidad
         axes[i].tick_params(axis='x', rotation=45)
 
   # Ocultar los axes adicionales si el numero de columnas categoricas no llena la ultima fila
   for j in range(i+1, num_filas * num_col):
     fig.delaxes(axes[j])
-
+    
   plt.tight_layout()  # Ajustar automaticamente los parametros de la subtrama
   plt.show()
-    
-def continuos(df):
+  
+  print('')
+  print('\033[1mTABLAS DE DATOS\033[0m')
+  print('')
+  if (nRows>1000):
+     tofile = True
+     
+  if not(tofile):
+     for i in arrayFrecuencia:
+        print(tabulate(i.to_frame(), headers='keys', tablefmt='fancy_grid'))
+        print('')
+  else:
+     
+     nombre_archivo = datetime.now().strftime("%Y%m%d_%H%M%S%f") + '.txt'  
+     print('Se obtuvieron Tablas extensas, los resultados se guardaron en el archivo: ', nombre_archivo)
+     with open(nombre_archivo, 'w') as f:
+        for i in arrayFrecuencia:
+           f.write('\n')
+           f.write(tabulate(i.to_frame(), headers='keys', tablefmt='simple'))
+           f.write('\n')
+     
+  
+def continuos(df, varint=False ):
   import warnings 
   warnings.filterwarnings('ignore')
   from tabulate import tabulate
   
+  if df.shape[1] < 1:
+     print("El DataFrame debe tener al menos una variable.")
+     return
+  
   pd.set_option('display.float_format', lambda x: '%.3f' % x)
   oLista = []
   oIndex = []
@@ -222,9 +302,17 @@ def continuos(df):
   
   # Variables decimal o float
   colNum = []
-  for i in df.columns:
-    if (df[i].dtype.kind in 'df'):
-        colNum.append(i)
+  
+  if (varint==True):
+      for i in df.columns:
+        # int, decimal o float
+        if (df[i].dtype.kind in 'idf'):
+            colNum.append(i)
+  else:
+      for i in df.columns:
+        # decimal o float
+        if (df[i].dtype.kind in 'df'):
+            colNum.append(i)
         
   # Selecciona solo las columnas numericas
   df = df[colNum]
@@ -290,19 +378,19 @@ def continuos(df):
 
   print('')
   print('\033[1mHISTOGRAMAS\033[0m')
-  histograma(df)
+  __histograma(df)
   
   print('')
   print('\033[1mGRAFICOS BOXPLOT\033[0m')
-  boxplot2(df, df.columns)
+  __boxplot(df, df.columns)
 
   print('')
   print('\033[1mGRAFICOS DE DISPERSION\033[0m')
-  dispersion_index(df)
+  __dispersion_index(df)
 
   print('')
   print('\033[1mGRAFICOS XY\033[0m')
-  graficosxy(df)
+  __graficosxy(df)
   plt.show()
   plt.rcParams['figure.figsize']=(6,6)
 
