@@ -15,6 +15,7 @@ class TestCategorical(unittest.TestCase):
 
         # Definir las categoras posibles
         categories = ['A', 'B', 'C', 'D']
+        
 
         # Generar datos categricos aleatorios
         data = []
@@ -24,7 +25,7 @@ class TestCategorical(unittest.TestCase):
         # Crear el DataFrame con los datos categricos
         df = pd.DataFrame(data, columns=variable_names)
 
-        eda.categorical(df)
+        eda.categorical(df,True)
         
 
 if __name__ == '__main__':
